--------------------------------------------------------------------------------
-- Weapon Sentry
-- Original Carnage Contest Weapon
-- Script by DC, September 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.enemy_rifle={}
cc.enemy_rifle.sen={}
cc.enemy_rifle.bullet={}
lol=1

-- Load & Prepare Ressources
cc.enemy_rifle.sfx_bounce=loadsfx("fall0.ogg")										-- Bounce
cc.enemy_rifle.gfx_wpn=loadgfx("ZJ/enemy_riflebody.png")								-- Sentry
setmidhandle(cc.enemy_rifle.gfx_wpn)
cc.enemy_rifle.gfx_gun=loadgfx("ZJ/enemy_rifle.png")							-- Gun
setmidhandle(cc.enemy_rifle.gfx_gun)
cc.enemy_rifle.gfx_pro=loadgfx("weapons/shot.bmp")									-- Projectile Image
setmidhandle(cc.enemy_rifle.gfx_pro)
cc.enemy_rifle.sfx_attack=loadsfx("rifle.wav")										-- Attack Sound

--------------------------------------------------------------------------------
-- Projectile: Bullet
--------------------------------------------------------------------------------

cc.enemy_rifle.bullet.id=addprojectile("cc.enemy_rifle.bullet")			-- Add Projectile

function cc.enemy_rifle.bullet.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,0)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.enemy_rifle.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.enemy_rifle.bullet.update(id)							-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.009
	projectiles[id].sy=projectiles[id].sy+getgravity()*0.45
	-- Move
	cc.enemy_rifle.bullet.move(id)
end

function cc.enemy_rifle.bullet.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20,1,1,1,-1,projectiles[id].ignore)==1 then
			-- Cause damage
			if playercollision()~=0 then
				playerpush(playercollision(),projectiles[id].sx/15.0,projectiles[id].sy/15.0)
				playerdamage(playercollision(),30)
				blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
			elseif objectcollision()>0 then
				objectdamage(objectcollision(),30)
			end
			-- Effects
			playsound(sfx_ricochet1)
			particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
			particlefadealpha(0.006)
			particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end

--------------------------------------------------------------------------------
-- Object: Sentry
--------------------------------------------------------------------------------

cc.enemy_rifle.sen.id=addobject("cc.enemy_rifle.sen",cc.enemy_rifle.gfx_wpn)			-- Add Object

function cc.enemy_rifle.sen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].sy=0
	objects[id].timer=0
	objects[id].burst=0
	objects[id].burstrow=0
	objects[id].health=50
	objects[id].dir=1
	objects[id].owner=playercurrent()
end

function cc.enemy_rifle.sen.draw(id,x,y)
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(0)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	-- Draw Sentry
	drawimage(cc.enemy_rifle.gfx_wpn,x,y)
	-- Draw Gun
	setalpha(1)
	setscale(objects[id].dir,1)
	drawimage(cc.enemy_rifle.gfx_gun,x+(objects[id].dir*4),y)
end

function cc.enemy_rifle.sen.update(id,x,y)
	-- Kill
	if objects[id].health<=0 then
		playsound(sfx_splatter1)
		particle(p_ring,x,y)
		particlecolor(255,0,0)
        particle(p_bodypart,x,y)
        particlecolor(255,255,0)
        for i = 1,3 do
        blood(x,y)
        end
        removeobject(id)
        return
	end
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.enemy_rifle.gfx_wpn,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(cc.enemy_rifle.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()+5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove
			removeobject(id)
			return
		end
	end
	objectposition(id,x,y)
	-- Scan for Enemies / Shoot
	if objects[id].burst==0 then
		objects[id].timer=objects[id].timer+1
		if objects[id].timer>=55 then
			-- Scan all players
			objects[id].timer=0
			local players=playertable(0,0)
			local ownally=getplayeralliance(objects[id].owner)
			for i=1,#players do
				-- Enemy?
				if lol==1 then
					-- In Range?
					if math.abs(getplayery(players[i])-y)<30 then
						if math.abs(getplayerx(players[i])-x)<720 then
							-- Righ Direction?
							if (objects[id].dir==1 and getplayerx(players[i])>x) or (objects[id].dir==-1 and getplayerx(players[i])<x) then
								-- Something blocking?
								local dist=(math.abs(getplayerx(players[i])-x))-8
								local tx=x+objects[id].dir*14
								local ok=1
								for offsetx=1,dist,5 do
									if collision(col3x3,tx+offsetx*objects[id].dir,y-2,1,1,1,players[i])==1 then
										ok=0; break;
									end
								end
								if ok==1 then
									-- Ready to fire!
									objects[id].burst=1
									objects[id].burstrow=objects[id].burstrow+1
								end
							elseif (getplayerx(players[i])>x) then
								objects[id].dir=1
							elseif (getplayerx(players[i])<x) then
								objects[id].dir=-1
							end
						end
					end
				end
			end
			if objects[id].burst==0 then
				objects[id].burstrow=0
			end
		end
	else
		-- Shoot!
		playsound(cc.enemy_rifle.sfx_attack)
		local pid=createprojectile(cc.enemy_rifle.bullet.id)
		projectiles[pid]={}
		-- Ignore Object
		projectiles[pid].ignore=id
		-- Set initial position of projectile
		projectiles[pid].x=x+objects[id].dir*18
		projectiles[pid].y=y+1
		-- Set speed of projectile
		math.randomseed(objects[id].burst*12345)
		offset=-6.0+math.random()*12.0
		projectiles[pid].sx=math.sin(math.rad(90*objects[id].dir))*16.0
		projectiles[pid].sy=-math.cos(math.rad(90*objects[id].dir))*16.0
		-- Initial movement
		projectiles[pid].x=projectiles[pid].x-objects[id].dir*20
		cc.enemy_rifle.bullet.move(pid)
		-- Effects
		particle(p_muzzle,projectiles[pid].x,projectiles[pid].y)
		particle(p_smoke,projectiles[pid].x,projectiles[pid].y)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		particle(p_bullet,x,y-7)
		objects[id].burst=0
		objects[id].timer=0
	end
end

function cc.enemy_rifle.sen.damage(id,damage)
	if objects[id].health>0 then
		objects[id].health=objects[id].health-damage
		if (damage>=1 and damage<=5) then
			particle(p_bigblood,getobjectx(id),getobjecty(id))
			particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
		elseif (damage>=6 and damage<=10) then
			for i = 1,2 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=11 and damage<=15) then
			for i = 1,3 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=16 and damage<=20) then
			for i = 1,4 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=21 and damage<=25) then
			for i = 1,5 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=26 and damage<=30) then
			for i = 1,6 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=36 and damage<=40) then
			for i = 1,7 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=41 and damage<=45) then
			for i = 1,7 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=26) then
			for i = 1,8 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		end
	end
end