--------------------------------------------------------------------------------
-- Straw Dummy
-- Original Carnage Contest Object
-- Script by DC, August 2011, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.friend_panic_bomb={}

-- Load & Prepare Ressources
cc.friend_panic_bomb.gfx=loadgfx("ZJ/friend_scream1.png")						-- Straw dummy graphic
sethandle(cc.friend_panic_bomb.gfx,11,15)
cc.friend_panic_bomb.gfx2=loadgfx("ZJ/friend_scream2.png")						-- Straw dummy graphic
sethandle(cc.friend_panic_bomb.gfx2,11,15)
cc.friend_panic_bomb.gfx_bomb=loadgfx("weapons/belt.bmp")
setmidhandle(cc.friend_panic_bomb.gfx_bomb)
cc.friend_panic_bomb.arrow=0													-- Draw Arrow (setting 1/0)

--------------------------------------------------------------------------------
-- Object: Straw Dummy
--------------------------------------------------------------------------------

cc.friend_panic_bomb.id=addobject("cc.friend_panic_bomb",cc.friend_panic_bomb.gfx)			-- Add Object
function cc.friend_panic_bomb.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=50
	objects[id].hurt=0
	objects[id].jump=0
	objects[id].movetimer=0
	objects[id].move=0
	objects[id].timer=0
	objects[id].sy=0
end
function cc.friend_panic_bomb.draw(id,x,y)
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setrotation(0)
	if objects[id].move==0 then
		if getplayerx(0)>x then
			setscale(1,1)
		else
			setscale(-1,1)
		end
	else
		setscale(objects[id].move,1)
	end
	-- Draw
	if objects[id].hurt>=1 then
		drawimage(cc.friend_panic_bomb.gfx2,x,y)
	else
		drawimage(cc.friend_panic_bomb.gfx,x,y)
	end
	-- Bomb
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setrotation(0)
	if objects[id].move==0 then
		if getplayerx(0)>x then
			setscale(1,1)
		else
			setscale(-1,1)
		end
	else
		setscale(objects[id].move,1)
	end
	drawimage(cc.friend_panic_bomb.gfx_bomb,x,y+8)
end
function cc.friend_panic_bomb.update(id,x,y)
	if objects[id].health<=0 then
		playsound(sfx_splatter1)
		for i = 1,3 do
			blood(x,y)
        end
		-- Explode
		terraincircle(x,y,30,0x00000000)
		terrainexplosion(getobjectx(id),getobjecty(id),0,3)
		particle(p_ring,x,y)
		particlesize(0.3,0.3)
		arealdamage(x,y,65,100)
		-- Crater
		grey=math.random(0,25)
		terrainalphaimage(gfx_crater150,x,y,math.random(6,9)*0.1,grey,grey,grey)
		removeobject(id)
        return
	elseif objects[id].health>=1 then
		if objects[id].timer>=500 then
			objects[id].health=0
		else
			objects[id].timer=objects[id].timer+1
		end
	end
	-- Countdown
	objects[id].hurt=objects[id].hurt-1
	-- Jump
	if objects[id].jump>=60 then
		if collision(colplayer,x,y+16,1,1,1,-1,id)==1 then
			objects[id].sy=objects[id].sy-math.random(2.0,3.0)
		end
		objects[id].jump=0
	else
		objects[id].jump=objects[id].jump+1
	end
	-- Move
	if objects[id].movetimer>=100 then
		objects[id].move=math.random(-1,1)
		objects[id].movetimer=0
	else
		objects[id].movetimer=objects[id].movetimer+1
	end
	if objects[id].move==1 then
		if collision(colplayer,x+11,y,1,1,1,-1,id)==0 then
			x=x+objects[id].move*0.7
		end
	elseif objects[id].move==-1 then
		if collision(colplayer,x-11,y,1,1,1,-1,id)==0 then
			x=x+objects[id].move*0.7
		end
	end
	-- Fall
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.friend_panic_bomb.gfx,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(loadsfx("fall0.ogg")) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()+5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove
			removeobject(id)
			break
		end
	end
	objectposition(id,x,y)
end
function cc.friend_panic_bomb.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		objects[id].hurt=18
		if (damage>=1 and damage<=5) then
			particle(p_bigblood,getobjectx(id),getobjecty(id))
			particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
		elseif (damage>=6 and damage<=10) then
			for i = 1,2 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=11 and damage<=15) then
			for i = 1,3 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=16 and damage<=20) then
			for i = 1,4 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=21 and damage<=25) then
			for i = 1,5 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=26 and damage<=30) then
			for i = 1,6 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=36 and damage<=40) then
			for i = 1,7 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=41 and damage<=45) then
			for i = 1,7 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		elseif (damage>=26) then
			for i = 1,8 do
				particle(p_bigblood,getobjectx(id),getobjecty(id))
				particlespeed(math.random(-1.5,1.5),math.random(-4,-2))
			end
		end
	end
end