--------------------------------------------------------------------------------
-- Weapon Pistol
-- Edited Carnage Contest Weapon
-- Script by ZJ, April 20012, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.zjpistol={}
cc.zjpistol.bullet={}

-- Load & Prepare Ressources
cc.zjpistol.gfx_wpn=loadgfx("weapons/pistol.bmp")					-- Weapon Image
setmidhandle(cc.zjpistol.gfx_wpn)
cc.zjpistol.gfx_pro=loadgfx("weapons/shot.bmp")					-- Projectile Image
setmidhandle(cc.zjpistol.gfx_pro)
cc.zjpistol.sfx_attack=loadsfx("pistol.wav")						-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Pistol
--------------------------------------------------------------------------------

cc.zjpistol.id=addweapon("cc.zjpistol","Pistol",cc.zjpistol.gfx_wpn,100,1,"ballistic")	-- Add Weapon
cc.zjpistol.ammo=1												-- 6 Bullets

function cc.zjpistol.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.zjpistol.gfx_wpn,6,0)
	-- HUD Crosshair
	if cc.zjpistol.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.zjpistol.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.zjpistol.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(1)
		-- Reset Timer
		weapon_timer=13
		-- Attack
		playsound(cc.zjpistol.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.zjpistol.bullet.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(5*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+2+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*16.5
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*16.5
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.zjpistol.bullet.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*5)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*5)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		particle(p_bullet,getplayerx(0)+(getplayerdirection(0)*2),getplayery(0)+2)
		-- End Turn
		if (weapon_shots>=cc.zjpistol.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bullet
--------------------------------------------------------------------------------

cc.zjpistol.bullet.id=addprojectile("cc.zjpistol.bullet")		-- Add Projectile

function cc.zjpistol.bullet.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,0)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.zjpistol.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.zjpistol.bullet.update(id)						-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.02
	projectiles[id].sy=projectiles[id].sy+getgravity()*0.75
	-- Move
	cc.zjpistol.bullet.move(id)
end

function cc.zjpistol.bullet.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					playerpush(playercollision(),projectiles[id].sx/10.0,projectiles[id].sy/10.0)
					playerdamage(playercollision(),2)
					blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),2)
				end
				-- Effects
				playsound(sfx_ricochet1)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end