--------------------------------------------------------------------------------
-- Weapon German Rifle
-- Edited Carnage Contest Weapon
-- Script by ZJ, April 20012, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.rifle={}
cc.rifle.bullet={}

-- Load & Prepare Ressources
cc.rifle.gfx_wpn=loadgfx("ZJ/rifle.bmp")					-- Weapon Image
setmidhandle(cc.rifle.gfx_wpn)
cc.rifle.gfx_pro=loadgfx("weapons/shot.bmp")					-- Projectile Image
setmidhandle(cc.rifle.gfx_pro)
cc.rifle.sfx_attack=loadsfx("rifle.wav")						-- Attack Sound
cc.rifle.sfx_waste=loadsfx("rifle2.wav")						-- Wasted Bullet Sound

--------------------------------------------------------------------------------
-- Weapon: German Rifle
--------------------------------------------------------------------------------

cc.rifle.id=addweapon("cc.rifle","Rifle",cc.rifle.gfx_wpn,100,1,"ballistic")	-- Add Weapon
cc.rifle.ammo=1												-- 6 Bullets

function cc.rifle.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.rifle.gfx_wpn,6,0)
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,20)
	end
	-- HUD Crosshair
	if cc.rifle.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.rifle.attack(attack)								-- Attack
	if (weapon_shots<=0) then
		-- Charge
		if (attack==1 and weapon_charge<=20) then
			-- No more weapon switching!
			useweapon(1)
			weapon_charge=weapon_charge+1										-- Increase charge
		end
		if (attack==0 and weapon_charge>0 and weapon_charge<20) then
			useweapon(1)
			-- Waste bullet
			playsound(cc.rifle.sfx_waste)
			weapon_shots=weapon_shots+1
			particle(p_bullet,getplayerx(0)+(getplayerdirection(0)*3),getplayery(0)+3)
			particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*3),getplayery(0)+3)
			particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
			particlefadealpha(0.005)
			endturn()
		end
		-- Fire a projectile (full charge)
		if  (attack==0 and weapon_charge>=20) then
			-- No more weapon switching!
			useweapon(1)
			-- Attack
			playsound(cc.rifle.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.rifle.bullet.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
			projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*16.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*16.0
			-- Initial movement
			projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
			projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
			for i=1,3,1 do
				if cc.rifle.bullet.move(id)==1 then
					break
				end
			end
			-- Effects
			recoil(4)
			particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
			particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
			particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
			particlefadealpha(0.005)
			particle(p_bullet,getplayerx(0)+(getplayerdirection(0)*3),getplayery(0)+3)
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bullet
--------------------------------------------------------------------------------

cc.rifle.bullet.id=addprojectile("cc.rifle.bullet")		-- Add Projectile

function cc.rifle.bullet.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,0)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.rifle.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.rifle.bullet.update(id)						-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.009
	projectiles[id].sy=projectiles[id].sy+getgravity()*0.45
	-- Move
	cc.rifle.bullet.move(id)
end

function cc.rifle.bullet.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					playerpush(playercollision(),projectiles[id].sx/15.0,projectiles[id].sy/15.0)
					playerdamage(playercollision(),35)
					blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),35)
				end
				-- Effects
				playsound(sfx_ricochet1)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end