--------------------------------------------------------------------------------
-- Straw Dummy
-- Original Carnage Contest Object
-- Script by DC, August 2011, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.strawdummy={}

-- Load & Prepare Ressources
cc.strawdummy.gfx=loadgfx("stuff/strawdummy.bmp")						-- Straw dummy graphic
cc.strawdummy.arrow=1													-- Draw Arrow (setting 1/0)
sethandle(cc.strawdummy.gfx,10,30)
cc.strawdummy.sfx=loadsfx("strawdummyburst.wav")

--------------------------------------------------------------------------------
-- Object: Straw Dummy
--------------------------------------------------------------------------------

cc.strawdummy.id=addobject("cc.strawdummy",cc.strawdummy.gfx)			-- Add Object
function cc.strawdummy.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=10
	objects[id].sy=0
end
function cc.strawdummy.draw(id,x,y)
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(math.sin(id*10+os.clock()*2.0)*10.0)
	-- Draw
	drawimage(cc.strawdummy.gfx,x,y)
	-- Arrow
	if cc.strawdummy.arrow==1 then
		outofscreenarrow(x,y)
	end
end
function cc.strawdummy.update(id,x,y)
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.strawdummy.gfx,x,y,1,1,1,-1,id)==1 then
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()+5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove
			removeobject(id)
			break
		end
	end
	objectposition(id,x,y)
end
function cc.strawdummy.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy
			playsound(cc.strawdummy.sfx)
			-- Dust & Particle
			particle(p_smoke,x,y-3)
			particlecolor(255,210,5)
			particlesize(16,16)
			particlealpha(2)
			particlespeed(0.0,-0.7)
			particlefadealpha(0.03)
			for i = 1 , math.random(2,3) do
				particle(p_bullet,x,y)
				particlecolor(255,210,5)
			end
			for i = 1 , math.random(2,3) do
				particle(p_bullet,x,y)
				particlecolor(230,170,5)
			end
			for i = 1 , math.random(2,3) do
				particle(p_bullet,x,y)
				particlecolor(205,135,35)
			end
			-- Remove
			removeobject(id)
		end
	end
end